/*
 * Copyright 2007 Ask Hjorth Larsen
 *
 * asklarsen@gmail.com
 *
 * This file is part of Wires.
 * 
 * Wires is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Wires is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Wires.  If not, see <http://www.gnu.org/licenses/>.
 */

package jwires;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 *
 * @author ask
 */
public class WireFrame extends JFrame {
    
    private Board board;
    private JPanel panel;
    private Main main;
    
    private final SettingsFrame settingsFrame = new SettingsFrame();
    
    private final JButton[] shiftButtons = {new JButton("Right"), 
            new JButton("Up"), new JButton("Left"), new JButton("Down")};
    
    final String license = 
            "<html>Wires is free software; you can redistribute it and/or modify<p>"+
            "it under the terms of the GNU General Public License as published by<p>"+
            "the Free Software Foundation; either version 3 of the License, or<p>"+
            "(at your option) any later version.<p><p>"+

            "Wires is distributed in the hope that it will be useful,<p>"+
            "but WITHOUT ANY WARRANTY; without even the implied warranty of<p>"+
            "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the<p>"+
            "GNU General Public License for more details.<p><p>"+

            "You should have received a copy of the GNU General Public License<p>"+
            "along with Wires.  If not, see <a href=\"http://www.gnu.org/licenses/\">http://www.gnu.org/licenses/</a></html>.";
    
    final String[] helpItems = {"About", "Instructions", "License"};
    final String[] helpTexts = {
        "<html><b>Wires</b><p><p>Rotate tiles until they form a (simply) connected graph.<p>Written by Ask Hjorth Larsen.<p>asklarsen@gmail.com</html>",
        "<html>Use the mouse scroll-wheel to rotate tiles until they are all connected.<p>" +
                "Right-click to flood-fill all connected tiles. Left-click on a tile<p>" +
                "to lock it, signifying (to yourself) that this tile is correctly aligned.</html>",
        license
    };
    
    
    /** Creates a new instance of WireFrame */
    public WireFrame(final Main main) {
        super("Wires");
        setResizable(false);
        this.main = main;
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        panel = new JPanel();
        panel.setLayout(new BorderLayout());
        add(panel);
        
        JMenuBar bar = new JMenuBar();
        
        JMenu gameMenu = new JMenu("Game");
        JMenu helpMenu = new JMenu("Help");
        bar.add(gameMenu);
        bar.add(helpMenu);
        setJMenuBar(bar);
        JToolBar toolBar = new JToolBar();
        toolBar.setFloatable(false);
        
        //JButton[] buttons = shiftButtons;
        for(int i = 0; i < 4; i++){
            final int direction = i;
            shiftButtons[i].addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent e){
                    main.map.shift(direction);
                    panel.repaint();
                }
            });
        }
        toolBar.add(shiftButtons[1]);
        toolBar.add(shiftButtons[3]);
        toolBar.add(shiftButtons[2]);
        toolBar.add(shiftButtons[0]);
        
        add(toolBar, BorderLayout.NORTH);
        
        for(int i = 0; i < helpItems.length; i++){
            final int j = i;
            makeItem(helpMenu, helpItems[i], new ActionListener(){
                int index = j;
                public void actionPerformed(ActionEvent e){
                    JOptionPane.showMessageDialog(WireFrame.this, new JLabel(helpTexts[index]), helpItems[index],
                            JOptionPane.INFORMATION_MESSAGE);
                }
            });
        }
        
        makeItem(gameMenu, "New", new ActionListener(){
            public void actionPerformed(ActionEvent e){
                settingsFrame.setVisible(true);
            }
        });
        
        gameMenu.addSeparator();
        
        makeItem(gameMenu, "Exit",new ActionListener(){
            public void actionPerformed(ActionEvent e){
                System.exit(0);
            }
        });        
    }
    
    public void makeItem(JMenu menu, String name, ActionListener action){
        JMenuItem item = new JMenuItem(name);
        item.addActionListener(action);
        menu.add(item);
    }
    
    public void setBoard(Board board){
        settingsFrame.widthSpinner.setValue(new Integer(main.map.width));
        settingsFrame.heightSpinner.setValue(new Integer(main.map.height));
        settingsFrame.seedSpinner.setValue(new Long(main.generator.getSeed()));
        
        this.board = board;
        panel.removeAll();
        panel.add(board, BorderLayout.CENTER);
        for(int i = 0; i < shiftButtons.length; i++){
            shiftButtons[i].setEnabled(main.map.topology.isShiftable(i));
        }
        
        pack();
    }
    
    private class SettingsFrame extends JFrame implements ChangeListener, ActionListener {
        
        final JTextField optionField = new JTextField();
        final JSpinner widthSpinner = new JSpinner(), heightSpinner = new JSpinner();
        final JSpinner seedSpinner = new JSpinner();
        final JCheckBox customSeedBox = new JCheckBox();
        final JCheckBox shuffleBox = new JCheckBox();
        final JComboBox topologyBox = new JComboBox(Topologies.values());
        
        public SettingsFrame(){
            super("New game");
            setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
            JPanel panel = new JPanel();
            panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
            add(panel);
            
            JPanel guiConfigPanel = new JPanel();
            guiConfigPanel.setBorder(BorderFactory.createTitledBorder("Game options"));
            guiConfigPanel.setLayout(new GridLayout(6, 2));
            panel.add(guiConfigPanel);
            
            topologyBox.setPreferredSize(new Dimension(0, 0));
            
            shuffleBox.setSelected(true);
            
            JPanel seedPanel = new JPanel();
            seedPanel.setLayout(new GroupLayout(seedPanel));
            //seedPanel.setLayout(new BoxLayout(seedPanel, BoxLayout.X_AXIS));
            seedPanel.add(customSeedBox);
            //setPreferredSize(new Dimension(Integer.MAX_VALUE, 0));
            
            JPanel seedSubPanel = new JPanel(new GridLayout(1,1));
            seedSubPanel.add(seedSpinner);
            
            seedPanel.add(seedSubPanel);
            
            customSeedBox.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent e){
                    boolean useCustomSeed = customSeedBox.isSelected();
                    seedSpinner.setEnabled(useCustomSeed);
                }
            });
            
            seedSpinner.setEnabled(customSeedBox.isSelected());
            
            JPanel p = guiConfigPanel;
            p.add(new JLabel("Width"));
            p.add(widthSpinner);
            p.add(new JLabel("Height"));
            p.add(heightSpinner);
            p.add(new JLabel("Shuffle"));
            p.add(shuffleBox);
            p.add(new JLabel("Topology"));
            p.add(topologyBox);
            p.add(new JLabel("Custom seed"));
            p.add(customSeedBox);
            p.add(new JLabel("Seed"));
            p.add(seedSpinner);
            
            /*
            seedPanel.setLayout(new GridLayout(1,2));//(seedPanel, BoxLayout.X_AXIS));
            seedPanel.setBorder(BorderFactory.createTitledBorder("Custom seed"));
            seedPanel.add(customSeedBox);
            seedPanel.add(seedSpinner);
            
            panel.add(seedPanel);*/
            
            widthSpinner.addChangeListener(this);
            heightSpinner.addChangeListener(this);
            shuffleBox.addActionListener(this);
            seedSpinner.addChangeListener(this);
            topologyBox.addActionListener(this);
            customSeedBox.addActionListener(this);
            
            optionField.setFont(new Font(Font.MONOSPACED, Font.PLAIN, 12));
            JPanel borderSubPanel = new JPanel(new GridLayout(1,1));
            borderSubPanel.setBorder(BorderFactory.createTitledBorder("Command line options"));
            borderSubPanel.add(optionField);
            
            optionField.setText(main.stdArgs);
            
            panel.add(borderSubPanel);
            
            JPanel buttonContainer = new JPanel();
            
            JButton createButton = new JButton("Create game");
            JButton cancelButton = new JButton("Cancel");

            panel.add(buttonContainer);
            
            ActionListener createListener = new ActionListener(){
                public void actionPerformed(ActionEvent e){
                    String optionText = optionField.getText();
                    String[] options = optionText.split("\\s");
                    String errMsg = null;
                    Main.GameOptions o;
                    try{
                        o = main.parseOptions(options);
                    } catch(NumberFormatException ex){
                        JOptionPane.showMessageDialog(SettingsFrame.this, 
                                "Cannot create game.\n\nBad option selection or command syntax.", 
                                "Error", JOptionPane.ERROR_MESSAGE);
                        return;
                    }                    
                    try{
                        if(o.width < 1 || o.height < 1){
                            errMsg = "Maybe you should try using POSITIVE width and height.";
                        } else if(o.width * o.height < 2){
                            errMsg = "Congratulations, you now have a connected graph with one vertex.";
                        } else {
                            main.startGame(o);
                            if(e.getSource() != optionField){
                                setVisible(false);
                            }
                        }
                    } catch(Exception ex){
                        ex.printStackTrace();
                        errMsg = "Unexpected error, stack trace written to stdout!";
                    }
                    if(errMsg != null){
                        JOptionPane.showMessageDialog(SettingsFrame.this, errMsg, "Error", 
                                JOptionPane.ERROR_MESSAGE);
                    }
                }
            };
            
            createButton.addActionListener(createListener);
            optionField.addActionListener(createListener);
            
            cancelButton.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent e){
                    setVisible(false);
                }
            });

            buttonContainer.add(createButton);
            buttonContainer.add(cancelButton);
            
            pack();
        }

        public void actionPerformed(ActionEvent e) {
            updateTextField();
        }

        public void stateChanged(ChangeEvent e) {
            updateTextField();
        }

        public void updateTextField(){
            int width = ((Integer)widthSpinner.getValue()).intValue();
            int height = ((Integer)heightSpinner.getValue()).intValue();
            boolean shuffle = shuffleBox.isSelected();
            long seed = ((Number)seedSpinner.getValue()).longValue();
            boolean useCustomSeed = customSeedBox.isSelected();
            String topology = topologyBox.getSelectedItem().toString();

            
            String command = ""+width+"x"+height+
                    (shuffle ? "" : " noshuffle")+
                    (useCustomSeed ? " seed:"+seed : "") + 
                    (topology.equals(Topologies.plane.toString()) ? "" : " topology:"+topology);

            optionField.setText(command);                
        }
    
    }
    
}
